@@ -39,7 +39,7 @@
 #endif
 
 #include <sys/types.h>
-#ifdef HAS_SHM
+#ifdef MITSHM
 #ifdef SVR4
 #include <sys/sysmacros.h>
 #endif
@@ -73,7 +73,7 @@
 static void XF86BigfontResetProc(ExtensionEntry *       /* extEntry */
     );
 
-#ifdef HAS_SHM
+#ifdef MITSHM
 
 /* A random signature, transmitted to the clients so they can verify that the
    shared memory segment they are attaching to was really established by the
@@ -126,7 +126,7 @@ CheckForShmSyscall(void)
 
 /* ========== Management of shared memory segments ========== */
 
-#ifdef HAS_SHM
+#ifdef MITSHM
 
 #ifdef __linux__
 /* On Linux, shared memory marked as "removed" can still be attached.
@@ -221,7 +221,7 @@ shmdealloc(ShmDescPtr pDesc)
 void
 XF86BigfontFreeFontShm(FontPtr pFont)
 {
-#ifdef HAS_SHM
+#ifdef MITSHM
     ShmDescPtr pDesc;
 
     /* If during shutdown of the server, XF86BigfontCleanup() has already
@@ -240,7 +240,7 @@ XF86BigfontFreeFontShm(FontPtr pFont)
 void
 XF86BigfontCleanup(void)
 {
-#ifdef HAS_SHM
+#ifdef MITSHM
     while (ShmList)
         shmdealloc(ShmList);
 #endif
@@ -277,7 +277,7 @@ ProcXF86BigfontQueryVersion(ClientPtr client)
         .minorVersion = SERVER_XF86BIGFONT_MINOR_VERSION,
         .uid = geteuid(),
         .gid = getegid(),
-#ifdef HAS_SHM
+#ifdef MITSHM
         .signature = signature,
         .capabilities = (client->local && !client->swapped)
                          ? XF86Bigfont_CAP_LocalShm : 0
@@ -329,7 +329,7 @@ ProcXF86BigfontQueryFont(ClientPtr client)
     int nCharInfos;
     int shmid;
 
-#ifdef HAS_SHM
+#ifdef MITSHM
     ShmDescPtr pDesc = NULL;
 #else
 #define pDesc 0
@@ -374,7 +374,7 @@ ProcXF86BigfontQueryFont(ClientPtr client)
     nUniqCharInfos = 0;
 
     if (nCharInfos > 0) {
-#ifdef HAS_SHM
+#ifdef MITSHM
         if (!badSysCall)
             pDesc = (ShmDescPtr) FontGetPrivate(pFont, FontShmdescIndex);
         if (pDesc) {
@@ -395,7 +395,7 @@ ProcXF86BigfontQueryFont(ClientPtr client)
                 pCI = xallocarray(nCharInfos, sizeof(xCharInfo));
                 if (!pCI)
                     return BadAlloc;
-#ifdef HAS_SHM
+#ifdef MITSHM
             }
 #endif
             /* Fill nCharInfos starting at pCI. */
@@ -427,7 +427,7 @@ ProcXF86BigfontQueryFont(ClientPtr client)
                     }
                 }
             }
-#ifdef HAS_SHM
+#ifdef MITSHM
             if (pDesc && !badSysCall) {
                 *(CARD32 *) (pCI + nCharInfos) = signature;
                 if (!xfont2_font_set_private(pFont, FontShmdescIndex, pDesc)) {
@@ -694,7 +694,7 @@ XFree86BigfontExtensionInit(void)
                      ProcXF86BigfontDispatch,
                      SProcXF86BigfontDispatch,
                      XF86BigfontResetProc, StandardMinorOpcode)) {
-#ifdef HAS_SHM
+#ifdef MITSHM
 #ifdef MUST_CHECK_FOR_SHM_SYSCALL
         /*
          * Note: Local-clients will not be optimized without shared memory
