@@ -34,12 +34,15 @@
 #include <netdb.h>
 #endif
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <X11/X.h>
 #include <X11/Xmd.h>
 #include "misc.h"
 #include "osdep.h"
+#include "xdmcp.h"
+#include "xdmauth.h"
 #include "input.h"
 #include "dixstruct.h"
 #include "opaque.h"
