@@ -1,4 +1,16 @@
-#!/bin/sh -e
+#!/bin/bash -e
+
+if test "x$XTEST_DIR" = "x"; then
+    echo "XTEST_DIR must be set to the directory of the xtest repository."
+    # Exit as a "skip" so make check works even without xtest.
+    exit 77
+fi
+
+if test "x$PIGLIT_DIR" = "x"; then
+    echo "PIGLIT_DIR must be set to the directory of the piglit repository."
+    # Exit as a "skip" so make check works even without piglit.
+    exit 77
+fi
 
 # this times out on Travis, because the tests take too long.
 if test "x$TRAVIS_BUILD_DIR" != "x"; then
@@ -17,13 +29,17 @@ weston --no-config --backend=headless-backend.so --socket=wayland-$$ &
 WESTON_PID=$!
 export WAYLAND_DISPLAY=wayland-$$
 
-# We can use either wayland-info or weston-info (deprecated), depending
-# on what's actually available.
-WAYLAND_INFO=wayland-info
-command -V $WAYLAND_INFO >/dev/null 2>&1 || WAYLAND_INFO=weston-info
+# Need to kill weston before exiting, or meson will time out waiting for it to terminate
+# We rely on bash's behaviour, which executes the EXIT trap handler even if the shell is
+# terminated due to receiving a signal
+trap 'kill $WESTON_PID' EXIT
 
 # Wait for weston to initialize before starting Xwayland
-timeout --preserve-status 60s bash -c "while ! $WAYLAND_INFO &>/dev/null; do sleep 1; done"
+if ! timeout 5s bash -c "while ! $XSERVER_BUILDDIR/hw/xwayland/Xwayland -pogo -displayfd 1 &>/dev/null; do sleep 1; done"; then
+    # Try running Xwayland one more time, so we can propagate its stdout/stderr
+    # output and exit status
+    $XSERVER_BUILDDIR/hw/xwayland/Xwayland -pogo -displayfd 1
+fi
 
 # Start an Xwayland server
 export PIGLIT_RESULTS_DIR=$XSERVER_BUILDDIR/test/piglit-results/xwayland
@@ -43,10 +59,4 @@ PIGLIT_ARGS="$PIGLIT_ARGS -x xsetfontpath@2"
 
 export PIGLIT_ARGS
 
-# Do not let run-piglit.sh exit status terminate this script prematurely
-set +e
 $XSERVER_DIR/test/scripts/run-piglit.sh
-PIGLIT_STATUS=$?
-
-kill $WESTON_PID
-exit $PIGLIT_STATUS
